package com.chelvc.framework.security.context;

import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.regex.Pattern;
import java.util.stream.Stream;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.util.StringUtils;
import com.fasterxml.jackson.core.type.TypeReference;
import org.springframework.util.CollectionUtils;

/**
 * 安全处理上下文工具类
 * <p>
 * 角色资源授权信息从Nacos配置中心获取并动态更新
 *
 * @author Woody
 * @date 2023/4/5
 */
public class SecurityContextHolder {
    /**
     * 权限属性名称
     */
    public static final String PERMISSION_PROPERTY = "platform.permission";

    /**
     * 方法资源拆分匹配模式
     */
    private static final Pattern METHOD_RESOURCE_SPLIT_PATTERN = StringUtils.getPattern(" +");

    /**
     * 权限类型引用
     */
    private static final TypeReference<Map<String, Set<String>>> PERMISSION_TYPE_REFERENCE =
            new TypeReference<Map<String, Set<String>>>() {
            };

    /**
     * 判断当前用户是否拥有指定资源权限
     *
     * @param method 请求方式
     * @param uri    资源地址
     * @return true/false
     */
    public static boolean hasPermission(String method, String uri) {
        if (StringUtils.isEmpty(method) || StringUtils.isEmpty(uri)) {
            return false;
        }

        // 基于用户角色及对应的资源判断是否拥有权限
        String[] roles = StringUtils.splitActives(SessionContextHolder.getAuthority(false));
        if (roles == null || roles.length == 0) {
            return false;
        }
        Map<String, Set<String>> permissions = ApplicationContextHolder.getSafeProperty(
                PERMISSION_PROPERTY, PERMISSION_TYPE_REFERENCE
        );
        if (CollectionUtils.isEmpty(permissions)) {
            return false;
        }
        return Stream.of(roles).anyMatch(role -> {
            Set<String> resources = permissions.get(role);
            return !CollectionUtils.isEmpty(resources) && resources.stream().anyMatch(resource -> {
                if (StringUtils.isEmpty(resource) || (resource = resource.trim()).isEmpty()) {
                    return false;
                }
                String[] sections = METHOD_RESOURCE_SPLIT_PATTERN.split(resource);
                if (sections.length == 1) {
                    return StringUtils.isPath(sections[0], uri);
                }
                return Objects.equals(method, sections[0]) && StringUtils.isPath(sections[1], uri);
            });
        });
    }
}
